@@ -1,8 +1,9 @@
 import dataclasses
+import importlib
 import inspect
 import re
+from collections import defaultdict
 from enum import Enum, auto
-from importlib.util import module_from_spec, spec_from_file_location
 from inspect import getfile
 from pathlib import Path
 from types import ModuleType
@@ -22,16 +23,16 @@ from typing import (
 from pydantic import NameEmail
 from sqlalchemy import Column, MetaData, Table
 from sqlalchemy.orm import declared_attr, registry
+from sqlalchemy.sql import ColumnCollection
 
 from amora.config import settings
 from amora.logger import logger
 from amora.protocols import Compilable, CompilableProtocol
-from amora.utils import list_files, model_path_for_target_path
+from amora.utils import ensure_path, list_files, model_path_for_target_path
 
 metadata = MetaData(schema=f"{settings.TARGET_PROJECT}.{settings.TARGET_SCHEMA}")
 mapper_registry = registry(metadata=metadata)
 
-
 Model = Type["AmoraModel"]
 Models = Iterable[Model]
 
@@ -105,6 +106,9 @@ class MaterializationTypes(AutoName):
     table = auto()
 
 
+Owner = NameEmail
+
+
 @dataclasses.dataclass
 class ModelConfig:
     """
@@ -123,7 +127,7 @@ class ModelConfig:
     partition_by: Optional[PartitionConfig] = None
     cluster_by: Optional[List[str]] = None
     labels: Labels = dataclasses.field(default_factory=set)
-    owner: Optional[NameEmail] = None
+    owner: Optional[Owner] = None
 
     @property
     def labels_dict(self) -> Dict[str, str]:
@@ -171,6 +175,17 @@ class AmoraModel:
             or re.sub(r"(?<!^)(?=[A-Z])", "_", cls.__mro__[0].__name__).lower()
         )
 
+    @classmethod
+    def columns(cls) -> Optional[ColumnCollection]:
+        if cls.__model_config__.materialized == MaterializationTypes.ephemeral:
+            cte = cls.source()
+            if cte is not None:
+                return cte.exported_columns
+            else:
+                return None
+        else:
+            return cls.__table__.columns
+
     @classmethod
     def dependencies(cls) -> List[Table]:
         source = cls.source()
@@ -194,10 +209,10 @@ class AmoraModel:
         return None
 
     @classmethod
-    def target_path(cls, model_file_path: Union[str, Path]) -> Path:
+    def target_path(cls) -> Path:
         # {settings.dbt_models_path}/a_model/a_model.py -> a_model/a_model.py
         strip_path = settings.models_path.as_posix()
-        relative_model_path = str(model_file_path).split(strip_path)[1][1:]
+        relative_model_path = str(cls.path()).split(strip_path)[1][1:]
         # a_model/a_model.py -> ~/project/amora/target/a_model/a_model.sql
         target_file_path = settings.target_path.joinpath(
             relative_model_path.replace(".py", ".sql")
@@ -206,7 +221,7 @@ class AmoraModel:
         return target_file_path
 
     @classmethod
-    def model_file_path(cls) -> Path:
+    def path(cls) -> Path:
         return Path(getfile(cls))
 
     @classmethod
@@ -230,22 +245,12 @@ def _is_amora_model(candidate: ModuleType) -> bool:
     )
 
 
+@ensure_path
 def amora_model_for_path(path: Path) -> Model:
-    spec = spec_from_file_location(".".join(["amoramodel", path.stem]), path)
-    if spec is None:
-        raise ValueError(f"Invalid path `{path}`. Not a valid Python file.")
-
-    module = module_from_spec(spec)
-
-    if spec.loader is None:
-        raise ValueError(f"Invalid AmoraModel path `{path}`. Unable to load module.")
-
     try:
-        spec.loader.exec_module(module)
-    except ImportError as e:
-        raise ValueError(
-            f"Invalid AmoraModel path `{path}`. Unable to load module."
-        ) from e
+        module = importlib.import_module(path.stem, settings.models_path.name)
+    except ModuleNotFoundError as e:
+        raise ValueError(f"Invalid path `{path}`") from e
 
     compilables = inspect.getmembers(
         module,
@@ -253,12 +258,7 @@ def amora_model_for_path(path: Path) -> Model:
     )
 
     for _name, class_ in compilables:
-        try:
-            # fixme: Quando carregamos o código em inspect, não existe um arquivo associado,
-            #  ou seja, ao iterar sobre as classes de um arquivo, a classe que retornar um TypeError,
-            #  é uma classe definida no arquivo
-            class_.model_file_path()
-        except TypeError:
+        if class_.path() == path:
             return class_
 
     raise ValueError(f"Invalid path `{path}`")
@@ -269,20 +269,11 @@ def amora_model_for_target_path(path: Path) -> Model:
     return amora_model_for_path(model_path)
 
 
-def model_path_for_model(model: Model) -> Path:
-    """
-    Returns the filepath where the model is defined.
-    """
-    for m, path in list_models():
-        if m.unique_name() == model.unique_name():
-            return path
-    raise FileNotFoundError("Model file not found in the project")
-
-
 def amora_model_for_name(model_name: str) -> Model:
-    for model, _path in list_models():
-        if model.unique_name() == model_name:
-            return model
+    for m in mapper_registry.mappers:
+        if m.class_.unique_name() == model_name:
+            return m.class_
+
     raise ValueError(f"{model_name} not found on models list")
 
 
@@ -292,8 +283,10 @@ def list_models(
     for model_file_path in list_files(path, suffix=".py"):
         if model_file_path.stem.startswith("_"):
             continue
+
         try:
             yield amora_model_for_path(model_file_path), model_file_path
+
         except ValueError:
             logger.exception(
                 "Unable to load amora model for path",
@@ -302,6 +295,24 @@ def list_models(
             continue
 
 
+def list_models_with_owner(owner: Union[str, Owner]) -> Iterable[Tuple[Model, Path]]:
+    if isinstance(owner, str):
+        owner = Owner.validate(owner)
+
+    for model, file_path in list_models():
+        if owner == model.__model_config__.owner:
+            yield model, file_path
+
+
+def owners_to_models_dict() -> Dict[str, List[Model]]:
+    owners_dict = defaultdict(list)
+    for model, _ in list_models():
+        owner = model.owner()
+        if owner:
+            owners_dict[owner].append(model)
+    return owners_dict
+
+
 def select_models_with_labels(labels: Labels) -> Iterable[Tuple[Model, Path]]:
     def matches_labels(item: Tuple[Model, Path]) -> bool:
         model, _model_path = item
